"""
Test for the new fixtures.
"""

import textwrap
from pathlib import Path

import pytest

pytest_plugins = ["pytester"]


def test_incorrectly_typed_parameter_fails(
    pytester: pytest.Pytester,
    tmp_path: Path,
) -> None:
    """
    With the plugin installed, a test with an incorrectly typed parameter
    fails.
    """
    test_file = tmp_path / "test_incorrectly_typed_parameter_fails.py"
    test_content = textwrap.dedent(
        text="""
        import pytest

        @pytest.mark.parametrize("x", [1, 2, 3])
        def test_all_fine(x: int):
            assert True

        @pytest.mark.parametrize("x", [1, "2", 3])
        def test_one_error(x: int):
            assert True
        """,
    )
    test_file.write_text(test_content)
    result = pytester.runpytest(test_file)
    result.assert_outcomes(passed=5, failed=1)


def test_method(
    pytester: pytest.Pytester,
    tmp_path: Path,
) -> None:
    """
    With the plugin installed, a test with an incorrectly typed parameter
    fails for a test in a class.
    """
    test_file = tmp_path / "test_incorrectly_typed_parameter_fails.py"
    test_content = textwrap.dedent(
        text="""
        import pytest

        class TestClass:
            @pytest.mark.parametrize("x", [1, 2, 3])
            def test_all_fine(self, x: int):
                assert True

            @pytest.mark.parametrize("x", [1, "2", 3])
            def test_one_error(self, x: int):
                assert True
        """,
    )
    test_file.write_text(test_content)
    result = pytester.runpytest(test_file)
    result.assert_outcomes(passed=5, failed=1)


def test_not_pytest_function(
    pytester: pytest.Pytester,
    tmp_path: Path,
) -> None:
    """
    The plugin doesn't error when something other than a ``pytest.Function``
    is given to ``pytest_collection_modifyitems``.
    """
    conftest_file = tmp_path / "conftest.py"
    conftest_file_content = textwrap.dedent(
        text="""
        from doctest import ELLIPSIS

        from sybil import Sybil
        from sybil.parsers.rest import (
            DocTestParser,
            PythonCodeBlockParser,
        )

        pytest_collect_file = Sybil(
            parsers=[
                DocTestParser(optionflags=ELLIPSIS),
                PythonCodeBlockParser(),
            ],
            patterns=["*.rst", "*.py"],
        ).pytest()

        """,
    )
    conftest_file.write_text(conftest_file_content)
    test_file = tmp_path / "test_example.rst"
    test_content = textwrap.dedent(
        text="""
        .. code-block:: python

           import os
        """,
    )
    test_file.write_text(test_content)
    result = pytester.runpytest(test_file)
    result.assert_outcomes(passed=1)
